@@ -6,21 +6,32 @@ paradigm.options <- OptionsManager('paradigm.options')
 
 
 # Adds guards to the base function for functional dispatching
-guard <- function(child.fn, condition, strict=TRUE)
+'%when%' <- function(child.fn, condition)
 {
+  strict <- TRUE
   child <- deparse(substitute(child.fn))
 
   expr <- deparse(substitute(condition))
-  # For debugging
-  #  cat("[From guard]\n")
-  #  cat("Parent: "); print(parent.frame())
-  #  cat("TopEnv: "); print(topenv(parent.frame()))
   if (length(grep('^(c\\()?function', expr, perl=TRUE)) < 1) 
     return(.guard(child, expr, strict, label='guard.xps'))
 
   return(.guard(child, condition, strict, label='guard.fns'))
 }
 
+# Deprecated
+guard <- function(child.fn, condition, strict=TRUE)
+{
+  cat("WARNING: This form is deprecated. Use the %when% operator instead\n")
+  child <- deparse(substitute(child.fn))
+
+  expr <- deparse(substitute(condition))
+  if (length(grep('^(c\\()?function', expr, perl=TRUE)) < 1) 
+    return(.guard(child, expr, strict, label='guard.xps'))
+
+  return(.guard(child, condition, strict, label='guard.fns'))
+}
+
+
 # Shortcut form for simple pattern matches
 # label := { guard.xps, guard.fns }
 .guard <- function(child, condition, strict, label)
@@ -104,14 +115,14 @@ isStrict <- function(child.fn)
   FALSE
 }
 
-isa <- function(type, argument)
+'%isa%' <- function(argument, type)
 {
   type <- gsub('[\'"]','',deparse(substitute(type)))
   type %in% class(argument)
 }
 
 # Note this will produce a vector of results
-hasa <- function(property, argument)
+'%hasa%' <- function(argument, property)
 {
   property <- gsub('[\'"]','',deparse(substitute(property)))
   property <- gsub(' ','', property, fixed=TRUE)
@@ -121,8 +132,7 @@ hasa <- function(property, argument)
   props %in% names(argument)
 }
 
-# If all properties exist
-hasall <- function(property, argument)
+'%hasall%' <- function(argument, property)
 {
   property <- gsub('[\'"]','',deparse(substitute(property)))
   property <- gsub(' ','', property, fixed=TRUE)
@@ -132,6 +142,7 @@ hasall <- function(property, argument)
   all(props %in% names(argument))
 }
 
+
 .SIMPLE_TYPES <- c('numeric','character','POSIXt','POSIXct')
 .is.simple <- function(x) any(class(x) %in% .SIMPLE_TYPES)
 .as.simple <- function(x)
@@ -214,6 +225,13 @@ UseFunction <- function(fn.name, ...)
   eval(parse(text=sprintf("%s(...)",fn.handle)))
 }
 
+    # Experimental to try to include default arguments
+    #if (length(args) > length(formals(f.exec)) ) next
+    # This is the number of arguments without default values
+    #required <- sapply(formals(f.exec), function(x) x == '')
+    #min.args <- sum(ifelse(required,1,0))
+    #if (length(args) < min.args) next
+
 .applyGuard <- function(guards, validator, ...)
 {
   args <- list(...)
@@ -293,4 +311,35 @@ AbuseMethod <- function(fn.name, type, ..., EXPLICIT=FALSE, ALWAYS=TRUE)
 }
 
 
+# Deprecated
+isa <- function(type, argument)
+{
+  type <- gsub('[\'"]','',deparse(substitute(type)))
+  type %in% class(argument)
+}
+
+# Deprecated
+# Note this will produce a vector of results
+hasa <- function(property, argument)
+{
+  property <- gsub('[\'"]','',deparse(substitute(property)))
+  property <- gsub(' ','', property, fixed=TRUE)
+  property <- sub('c(','', property, fixed=TRUE)
+  property <- sub(')','', property, fixed=TRUE)
+  props <- strsplit(property, ',', fixed=TRUE)[[1]]
+  props %in% names(argument)
+}
+
+# Deprecated
+# If all properties exist
+hasall <- function(property, argument)
+{
+  property <- gsub('[\'"]','',deparse(substitute(property)))
+  property <- gsub(' ','', property, fixed=TRUE)
+  property <- sub('c(','', property, fixed=TRUE)
+  property <- sub(')','', property, fixed=TRUE)
+  props <- strsplit(property, ',', fixed=TRUE)[[1]]
+  all(props %in% names(argument))
+}
+
 
